--
-- Table structure for table 'dasl_lockdiscovery'
--
-- Version 0.9.6

CREATE TABLE dasl_lock (
  URI blob NOT NULL,
  locktype int(1) NOT NULL, 	
  scope int(11) default NULL,
  depth int(11) default NULL,
  timeout bigint(20) NOT NULL default '0',
  locktoken varchar(255) default NULL,
  owner blob,
  auth_user blob,
  lockkey blob,
  KEY URIidx (URI(255))
) TYPE=MyISAM;

--
-- Table structure for table 'dasl_locknull'
--

CREATE TABLE dasl_locknull (
  path blob NOT NULL,
  fname blob,
  KEY pathidx (path(255)),
  UNIQUE KEY fnameidx (fname(255))
) TYPE=MyISAM;


--
-- Table structure for table 'dasl_namespace'
--
CREATE TABLE dasl_namespace (
  ns_id int(11) NOT NULL auto_increment,
  name varchar(255) NOT NULL default '',
  PRIMARY KEY  (ns_id),
  UNIQUE KEY nameidx (name)
) TYPE=MyISAM;

--
-- Table structure for table 'dasl_property'
--
CREATE TABLE dasl_property (
  serialno int(11) NOT NULL default '0',
  ns_id int(11) NOT NULL default '0',
  name varchar(255) NOT NULL default '',
  value blob,
  primary key (serialno, ns_id, name),
  KEY serialidx (serialno),
  KEY nvidx (name,value(245))
) TYPE=MyISAM;

--
-- Table structure for table 'dasl_resource'
--

CREATE TABLE dasl_resource (
  serialno int(11) NOT NULL auto_increment,
  URI blob NOT NULL,
  creationdate bigint(20) default NULL,
  displayname blob,
  getcontentlanguage varchar(20) default NULL,
  getcontentlength int(11) NOT NULL default '0',
  getcontenttype varchar(255) default NULL,
  getetag blob,
  getlastmodified bigint(20) default NULL,
  resourcetype int(11) default NULL,
  source int(11) default NULL,
  depth int(11) NOT NULL default '0',
  istext tinyint not null default 1,
  isexternal tinyint not null default 0,
  textcontent longtext,
  bincontent longblob,
  isversioned tinyint default 0,
  checkin int(11) default -1,
  checkout int(11) default -1,
  PRIMARY KEY  (serialno),
---
--- FIXME: Speed VS UNIQUE
--- UNIQUE KEY URIidx (URI(255)),
---
  UNIQUE KEY URIidx (URI(255)),
  KEY creationdateidx (creationdate),
  KEY displaynameidx (displayname(255)),
  KEY gcontlangidx (getcontentlanguage),
  KEY gcontlengidx (getcontentlength),
  KEY gconttypeidx (getcontenttype),
  KEY getetagidx (getetag(255)),
  KEY glstmodidx (getlastmodified),
  KEY restypeidx (resourcetype),
  KEY sourceidx (source),
  KEY depthidx (depth),
  FULLTEXT textcontentidx (textcontent)
) TYPE=MyISAM;

--
-- Table structure for table 'version_resource'
--

CREATE TABLE version_resource (
  serialno int(11) NOT NULL ,
  version integer not null,
  URI blob NOT NULL,
  creationdate bigint(20) default NULL,
  displayname blob,
  getcontentlanguage varchar(20) default NULL,
  getcontentlength int(11) NOT NULL default '0',
  getcontenttype varchar(255) default NULL,
  getetag blob,
  getlastmodified bigint(20) default NULL,
  resourcetype int(11) default NULL,
  source int(11) default NULL,
  depth int(11) NOT NULL default '0',
  istext tinyint not null default 1,
  isexternal tinyint(4) NOT NULL default '0',
  textcontent longtext,
  bincontent longblob,
  author varchar(50),
  PRIMARY KEY  (serialno, version),
---
--- FIXME: Speed VS UNIQUE
--- UNIQUE KEY URIidx (URI(255)),
---
  KEY URIidx (URI(255)),
  KEY creationdateidx (creationdate),
  KEY displaynameidx (displayname(255)),
  KEY gcontlangidx (getcontentlanguage),
  KEY gcontlengidx (getcontentlength),
  KEY gconttypeidx (getcontenttype),
  KEY getetagidx (getetag(255)),
  KEY glstmodidx (getlastmodified),
  KEY restypeidx (resourcetype),
  KEY sourceidx (source),
  KEY depthidx (depth),
  FULLTEXT textcontentidx (textcontent)
) TYPE=MyISAM;

--
-- Table structure for table 'version_property'
--

CREATE TABLE version_property (
  serialno int(11) NOT NULL default '0',
  version integer not null,
  ns_id int(11) NOT NULL default '0',
  name varchar(255) NOT NULL default '',
  value blob,
  primary key (serialno, version, ns_id, name),
  KEY serialidx (serialno, version),
  KEY nvidx (name,value(245))
) TYPE=MyISAM;

--
-- concat_substring()
--
CREATE FUNCTION CONCAT_SUBSTRING (str varchar(255), str2 varchar(255), pos integer) RETURNS varchar(255) CHARSET latin1
DETERMINISTIC
RETURN CONCAT(str, SUBSTRING(str2,pos));

--
-- create_temporary_coll_resource_table()
--
CREATE FUNCTION CREATE_TEMPORARY_COLL_RESOURCE_TABLE (
destUri VARCHAR(255),
srcUri VARCHAR(255),
len INTEGER,
vDepth INTEGER
) RETURNS int(11)
DETERMINISTIC
BEGIN
    
    DECLARE vUri VARCHAR(255);
    
        SET vUri = srcUri;
    SET vUri = CONCAT(vUri, '%');


    CREATE TEMPORARY TABLE tmp_resrecord SELECT 0, 
                    CONCAT_SUBSTRING(destUri, URI , len) AS uri_x, 
                    creationdate, 
                    displayname, 
                    getcontentlanguage, 
                    getcontentlength, 
                    getcontenttype, 
                    getetag, 
                    getlastmodified, 
                    resourcetype, 
                    source, 
                    depth + vDepth AS depth, 
                    istext, 
                    isexternal, 
                    textcontent, 
                    bincontent, 
                    0 AS isversioned, 
                    -1 AS checkin, 
                    -1 AS checkout 
                    FROM dasl_resource 
                    WHERE uri=srcUri OR uri LIKE vUri;

    RETURN 0;

END;

--
-- create_temporary_propterty_table()
--
CREATE FUNCTION CREATE_TEMPORARY_PROPERTY_TABLE (
    destSerialno INTEGER,
    srcSerialno INTEGER
) RETURNS int(11)
DETERMINISTIC
BEGIN
    CREATE temporary table tmp_resprops 
                               SELECT destSerialno, 
                               ns_id, 
                               name, 
                               value 
                               FROM dasl_property 
                               WHERE serialno = srcSerialno;
    RETURN 0;
END;

--
-- create_temporary_resource_table()
--
CREATE FUNCTION CREATE_TEMPORARY_RESOURCE_TABLE (
    vUri VARCHAR(255),
    vName VARCHAR(255),
    vDepth INTEGER,
    vSerialno INTEGER
) RETURNS int(11)
DETERMINISTIC
BEGIN
    CREATE TEMPORARY TABLE tmp_resrecord SELECT 0, 
        vUri AS uri, 
        creationdate, 
        vName AS displayname, 
        getcontentlanguage, 
        getcontentlength, 
        getcontenttype, 
        getetag, 
        getlastmodified, 
        resourcetype, 
        source, 
        vDepth AS depth, 
        istext, 
        isexternal, 
        textcontent, 
        bincontent, 
        0 AS isversioned, 
        -1 AS checkin, 
        -1 AS checkout 
        FROM dasl_resource 
        WHERE serialno=vSerialno;
    RETURN 0;
END;

--
-- create_temporary_version_property_table()
--
CREATE FUNCTION CREATE_TEMPORARY_VERSION_PROPERTY_TABLE (
    vVersion INTEGER,
    vSerialno INTEGER
) RETURNS int(11)
DETERMINISTIC
BEGIN
    CREATE TEMPORARY TABLE tmp_propertyforvcr 
                       SELECT dasl_property.serialno, 
                       vVersion AS version, 
                       ns_id, 
                       name, 
                       value 
                       FROM dasl_property, dasl_resource 
                       WHERE dasl_property.serialno=vSerialno 
                       AND dasl_property.serialno=dasl_resource.serialno;
    RETURN 0;
END;

--
-- create_temporary_version_resource_table()
--
CREATE FUNCTION CREATE_TEMPORARY_VERSION_RESOURCE_TABLE (vSerialno INTEGER,
vDate BIGINT(20),
vVersion INTEGER) RETURNS int(11)
DETERMINISTIC
BEGIN
    CREATE TEMPORARY TABLE tmp_resforvcr SELECT serialno, 
                       vVersion AS version, 
                       uri, 
                       vDate AS getcreationdate, 
                       displayname, 
                       getcontentlanguage, 
                       getcontentlength, 
                       getcontenttype, 
                       null AS getetag, 
                       vDate AS getlastmodified, 
                       resourcetype, 
                       source, 
                       depth,
                       istext, 
                       isexternal, 
                       textcontent, 
                       bincontent, 
                       'author' 
                       FROM dasl_resource 
                       WHERE serialno=vSerialno;
    RETURN 0;
END;

--
-- Fulltext search function for dasl_resource
--
CREATE FUNCTION `FT_SEARCH`
(id integer, txt varchar(255)) RETURNS double
DETERMINISTIC
BEGIN
    DECLARE val double;

    SELECT MATCH textcontent AGAINST ( txt ) INTO val FROM dasl_resource WHERE serialno = id;
    RETURN val;
END
