--
-- SQL script create table structures for Catacomb modDAV repository module
--
-- PostgreSQL version

--
-- Table structure for table 'dasl_lockdiscovery'
--

CREATE TABLE dasl_lock ( 
    uri      	varchar(4096) NOT NULL,
    locktype 	int2 NOT NULL,
    scope    	int4 NULL,
    depth    	int4 NULL,
    timeout  	int8 NOT NULL DEFAULT 0::bigint,
    locktoken	varchar(255) NULL,
    owner    	varchar NULL,
    auth_user	varchar(255) NULL,
    lockkey  	bytea NULL 
);

CREATE INDEX dasl_lock_uri_idx ON dasl_lock (uri);

--
-- Table structure for table 'dasl_locknull'
--

CREATE TABLE dasl_locknull (
    path	varchar(4096) NOT NULL,
    fname 	varchar(4096) UNIQUE
);

CREATE INDEX dasl_locknull_path_idx ON dasl_locknull (path);

--
-- Table structure for table 'dasl_namespace'
--
CREATE TABLE dasl_namespace (
  ns_id 	SERIAL PRIMARY KEY,
  name 		varchar(255) UNIQUE NOT NULL default ''
);

--
-- Table structure for table 'dasl_property'
--
CREATE TABLE dasl_property (
  serialno  	int8 NOT NULL default '0',
  ns_id     	int4 NOT NULL default '0',
  name      	varchar(255) NOT NULL default '',
  value     	varchar(4096),
  PRIMARY KEY (serialno, ns_id, name)
);

CREATE INDEX dasl_property_serialno_idx ON dasl_property ( serialno);
CREATE INDEX dasl_property_nv_idx ON dasl_property ( name, value);

--
-- Table structure for table 'dasl_resource'
--

CREATE TABLE dasl_resource (
  serialno 	BIGSERIAL PRIMARY KEY, 
  uri 		varchar(4096) UNIQUE, 
  creationdate 	int8 DEFAULT 0::bigint, 
  displayname 	varchar(255), 
  getcontentlanguage varchar(20) default NULL,
  getcontentlength int4 NOT NULL default '0',
  getcontenttype varchar(255) default NULL,
  getetag 	varchar(255),
  getlastmodified int8 default NULL,
  resourcetype 	int4 default NULL,
  source 	int4 default NULL,
  depth 	int4 NOT NULL default '0',
  istext 	int2 not null default 1,
  isexternal 	int2 not null default 0,
  textcontent 	text,
  bincontent 	bytea,
  isversioned 	int2 default 0,
  checkin 	int4 default -1,
  checkout 	int4 default -1
);

CREATE INDEX dasl_resource_creationdate_idx ON dasl_resource (creationdate);
CREATE INDEX dasl_resource_displayname_idx ON dasl_resource (displayname);
CREATE INDEX dasl_resource_getcontentlanguage_idx ON dasl_resource (getcontentlanguage);
CREATE INDEX dasl_resource_getcontentlength_idx ON dasl_resource (getcontentlength);
CREATE INDEX dasl_resource_gconttype_idx ON dasl_resource (getcontenttype);
CREATE INDEX dasl_resource_getetag_idx ON dasl_resource (getetag);
CREATE INDEX dasl_resource_getlastmodified_idx ON dasl_resource (getlastmodified);
CREATE INDEX dasl_resource_resourcetype_idx ON dasl_resource (resourcetype);
CREATE INDEX dasl_resource_source_idx ON dasl_resource (source);
CREATE INDEX dasl_resource_depth_idx ON dasl_resource (depth);

--
-- Table structure for table 'version_resource'
--

CREATE TABLE version_resource (
  serialno 	int8 NOT NULL ,
  version 	int4 not null,
  uri 		varchar(4096) NOT NULL,
  creationdate 	int8 default NULL,
  displayname 	varchar(255),
  getcontentlanguage varchar(20) default NULL,
  getcontentlength int8 NOT NULL default '0',
  getcontenttype varchar(255) default NULL,
  getetag 	varchar(255),
  getlastmodified int8 default NULL,
  resourcetype int4 default NULL,
  source 	int4 default NULL,
  depth 	int4 NOT NULL default '0',
  istext 	int2 not null default 1,
  isexternal 	int2 NOT NULL default '0',
  textcontent 	text,
  bincontent 	bytea,
  author 	varchar(50),
  PRIMARY KEY  (serialno, version)
);

CREATE INDEX version_resource_uri_idx ON version_resource (uri);
CREATE INDEX version_resource_creationdate_idx ON version_resource (creationdate);
CREATE INDEX version_resource_displayname_idx ON version_resource (displayname);
CREATE INDEX version_resource_getcontentlanguage_idx ON version_resource (getcontentlanguage);
CREATE INDEX version_resource_getcontentlength_idx ON version_resource (getcontentlength);
CREATE INDEX version_resource_gconttype_idx ON version_resource (getcontenttype);
CREATE INDEX version_resource_getetag_idx ON version_resource (getetag);
CREATE INDEX version_resource_getlastmodified_idx ON version_resource (getlastmodified);
CREATE INDEX version_resource_resourcetype_idx ON version_resource (resourcetype);
CREATE INDEX version_resource_source_idx ON version_resource (source);
CREATE INDEX version_resource_depth_idx ON version_resource (depth);

--
-- Table structure for table 'version_property'
--

CREATE TABLE version_property (
  serialno 	int8 NOT NULL default '0',
  version 	int4 not null,
  ns_id 	int4 NOT NULL default '0',
  name 		varchar(255) NOT NULL default '',
  value 	varchar(4096),
  primary key (serialno, version, ns_id, name)
);

CREATE INDEX version_property_sv_idx ON version_property ( serialno, version);
CREATE INDEX version_property_nv_idx ON version_property ( name, value);

--
-- LAST_INSERT_ID_BY_TABLENAME_AND_COLUMN() function
--

CREATE FUNCTION LAST_INSERT_ID_BY_TABLENAME_AND_COLUMN(varchar,varchar ) RETURNS int8
AS '
DECLARE
intable alias for $1;
incolumn alias for $2;
myvalue integer;

BEGIN

for myvalue in execute  '' select  currval('''' ''
|| intable || ''_'' || incolumn
|| ''_seq'''')
'' Loop
return myvalue;


END loop;
END;

'
language 'plpgsql';

--
-- CREATE_TEMPORARY_COLL_RESOURCE_TABLE() function
--

CREATE OR REPLACE FUNCTION "public"."create_temporary_coll_resource_table" (desturi varchar, srcuri varchar, len int4, vdepth int4) RETURNS integer AS
'
DECLARE
    vUri varchar;
    strLen varchar;
    strDepth varchar;
    stmt varchar;
BEGIN
    
    vUri := srcUri;
    vUri := vUri || ''%'';

    strLen := len;
    strDepth := vDepth;

    CREATE TEMPORARY TABLE tmp_resrecord(
        serialno          	int8 NOT NULL,
        uri               	varchar(4096) NULL,
        creationdate      	int8 NULL DEFAULT (0)::bigint,
        displayname       	varchar(255) NULL,
        getcontentlanguage	varchar(20) NULL,
        getcontentlength  	int4 NOT NULL DEFAULT 0,
        getcontenttype    	varchar(255) NULL,
        getetag           	varchar(255) NULL,
        getlastmodified   	int8 NULL,
        resourcetype      	int4 NULL,
        source            	int4 NULL,
        depth             	int4 NOT NULL DEFAULT 0,
        istext            	int2 NOT NULL DEFAULT 1,
        isexternal        	int2 NOT NULL DEFAULT 0,
        textcontent       	text NULL,
        bincontent        	bytea NULL,
        isversioned       	int2 NULL DEFAULT 0,
        checkin           	int4 NULL DEFAULT -1,
        checkout          	int4 NULL DEFAULT -1,
        PRIMARY KEY(serialno)
    );

    
    stmt := ''INSERT INTO tmp_resrecord SELECT nextval(''''dasl_resource_serialno_seq''''::regclass), ( '''''' || destUri || '''''' || SUBSTRING(uri, '' || strLen || '')) AS uri_x, '' ||
                        '' creationdate, displayname, getcontentlanguage, getcontentlength, getcontenttype, getetag, '' ||
                        '' getlastmodified, resourcetype, source, depth + '' || strDepth || '' AS depth, '' ||
                        '' istext, isexternal, textcontent, bincontent, 0 AS isversioned, -1 AS checkin, -1 AS checkout '' ||
                    ''FROM dasl_resource WHERE uri= '''''' || srcUri || '''''' OR uri LIKE '''''' || vUri || '''''' ;'';

    EXECUTE stmt;

    RETURN 0;

END
'
LANGUAGE 'plpgsql'

--
-- CREATE FUNCTION CREATE_TEMPORARY_RESOURCE_TABLE()
--

CREATE OR REPLACE FUNCTION "public"."create_temporary_resource_table" (vuri varchar, vname varchar, vdepth int4, vserialno int8) RETURNS integer AS
'
DECLARE
    strSerialno varchar;
    strDepth varchar;
    stmt varchar;
BEGIN
   
    strSerialno := vserialno;
    strDepth := vDepth;

    CREATE TEMPORARY TABLE tmp_resrecord(
        serialno          	int8 NOT NULL,
        uri               	varchar(4096) NULL,
        creationdate      	int8 NULL DEFAULT (0)::bigint,
        displayname       	varchar(255) NULL,
        getcontentlanguage	varchar(20) NULL,
        getcontentlength  	int4 NOT NULL DEFAULT 0,
        getcontenttype    	varchar(255) NULL,
        getetag           	varchar(255) NULL,
        getlastmodified   	int8 NULL,
        resourcetype      	int4 NULL,
        source            	int4 NULL,
        depth             	int4 NOT NULL DEFAULT 0,
        istext            	int2 NOT NULL DEFAULT 1,
        isexternal        	int2 NOT NULL DEFAULT 0,
        textcontent       	text NULL,
        bincontent        	bytea NULL,
        isversioned       	int2 NULL DEFAULT 0,
        checkin           	int4 NULL DEFAULT -1,
        checkout          	int4 NULL DEFAULT -1,
        PRIMARY KEY(serialno)
    );

    
    stmt := ''INSERT INTO tmp_resrecord SELECT nextval(''''dasl_resource_serialno_seq''''::regclass), '''''' || vuri || '''''' AS uri, '' ||
                        '' creationdate, '''''' || vname || '''''' AS displayname, getcontentlanguage, getcontentlength, getcontenttype, getetag, '' ||
                        '' getlastmodified, resourcetype, source, '' || strDepth || '' AS depth, '' ||
                        '' istext, isexternal, textcontent, bincontent, 0 AS isversioned, -1 AS checkin, -1 AS checkout '' ||
                    ''FROM dasl_resource WHERE serialno= '' || strSerialno || '';'';

    EXECUTE stmt;

    RETURN 0;

END
'
LANGUAGE 'plpgsql'

--
-- CREATE FUNCTION CREATE_TEMPORARY_PROPERTY_TABLE()
--

CREATE OR REPLACE FUNCTION "public"."create_temporary_property_table" (destSerialno int8, srcSerialno int8) RETURNS integer AS
'
DECLARE
    strDestSerialno varchar := destSerialno;
    strSrcSerialno varchar := srcSerialno;
    stmt varchar;
BEGIN
    CREATE TEMPORARY TABLE tmp_resprops (
            serialno	int4 NOT NULL DEFAULT 0,
            ns_id   	int4 NOT NULL DEFAULT 0,
            name    	varchar NOT NULL DEFAULT '''',
            value   	varchar(4096) NULL,
            PRIMARY KEY(serialno,ns_id,name)
    );

    stmt := ''INSERT INTO tmp_resprops SELECT '' || strDestSerialno || '' AS serialno, '' ||
	                           ''ns_id, name, value FROM dasl_property ''||
	                           ''WHERE serialno = '' || strSrcSerialno || '';'';

    EXECUTE stmt;

    RETURN 0;
END
'
LANGUAGE plpgsql

--
-- CREATE FUNCTION CREATE_TEMPORARY_VERSION_PROPERTY_TABLE()
--

CREATE OR REPLACE FUNCTION "public"."create_temporary_version_property_table" (vversion int4, vserialno int8) RETURNS int4 AS
'
DECLARE
    strVersion varchar := vVersion;
    strSerialno varchar := srcSerialno;
    stmt varchar;
BEGIN
    CREATE TEMPORARY TABLE tmp_propertyforvcr (
            serialno	int8 NOT NULL DEFAULT 0,
            version     int4 NOT NULL,
            ns_id   	int4 NOT NULL DEFAULT 0,
            name    	varchar NOT NULL DEFAULT '''',
            value   	varchar(4096) NULL,
            PRIMARY KEY(serialno,version,ns_id,name)
    );

    stmt := ''INSERT INTO tmp_resprops SELECT dasl_property.serialno, '' || strVersion || '' AS version, '' ||
                       '' ns_id, name, value FROM dasl_property, dasl_resource '' ||
                       '' WHERE dasl_property.serialno= '' || vSerialno ||
                       '' AND dasl_property.serialno=dasl_resource.serialno;'';

    EXECUTE 0;

    RETURN 0;
END
'
LANGUAGE 'plpgsql'

--
-- CREATE FUNCTION CREATE_TEMPORARY_VERSION_RESOURCE_TABLE()
--

CREATE OR REPLACE FUNCTION "public"."create_temporary_version_resource_table" (vSerialno int8, vDate int8, vVersion int4) RETURNS int4 AS
'
DECLARE
    strSerialno varchar := vSerialno;
    strDate varchar := vDate;
    strVersion varchar := vVersion;
    stmt varchar;
BEGIN

    CREATE TEMPORARY TABLE tmp_resforvcr(
        serialno          	int8 NOT NULL,
        version             int4 NOT NULL,
        uri               	varchar(4096) NULL,
        creationdate      	int8 NULL DEFAULT (0)::bigint,
        displayname       	varchar(255) NULL,
        getcontentlanguage	varchar(20) NULL,
        getcontentlength  	int4 NOT NULL DEFAULT 0,
        getcontenttype    	varchar(255) NULL,
        getetag           	varchar(255) NULL,
        getlastmodified   	int8 NULL,
        resourcetype      	int4 NULL,
        source            	int4 NULL,
        depth             	int4 NOT NULL DEFAULT 0,
        istext            	int2 NOT NULL DEFAULT 1,
        isexternal        	int2 NOT NULL DEFAULT 0,
        textcontent       	text NULL,
        bincontent        	bytea NULL,
        author              varchar(50) NULL,
        PRIMARY KEY(serialno)
    );

    
    stmt := ''INSERT INTO tmp_resforvcr SELECT serialno, '' || strVersion || '' AS version, uri, '' || strDate || '' AS getcreationdate, '' ||
                       '' displayname, getcontentlanguage, getcontentlength, getcontenttype, '' || 
                       '' null AS getetag, '' || strDate || '' AS getlastmodified, resourcetype, source, depth, istext, '' ||
                       ''isexternal, textcontent, bincontent, ''''author'''' AS author FROM dasl_resource WHERE serialno= '' || strSerialno || '';'';

    EXECUTE stmt;

    RETURN 0;

END
'
LANGUAGE 'plpgsql'

--
-- CREATE FUNCTION CONCAT_SUBSTRING()
--

CREATE OR REPLACE FUNCTION "public"."concat_substring" (str varchar, str2 varchar, pos int4) RETURNS varchar AS
'
BEGIN
RETURN str || SUBSTRING(str2, pos);
END
'
LANGUAGE 'plpgsql'